c ******************************************************* c
c
      subroutine calc_alpha_correction(zred, ibox, Lbox)
c
c ******************************************************* c
c
c     computing alpha correction used in propagators
c     from halo-fit
c
      implicit none
c
      integer  ik, ikmax, ik_max, ik_max_nl
      integer  iz, izmax, iz_max, ibox, ix, ixmax
      parameter(ikmax=5000, izmax=100, ixmax=500)
      real*8  ak(ikmax), pk(ikmax)
      real*8  ak_nl(ikmax), pk_nl(ikmax)
      real*8  z, zmax, zred, denom, numer, growth, Lbox, pi
      real*8  kmin, kmax, pklin, pknl
      real*8  eta(izmax), alphaz(izmax), rknl
      real*8  wk(ixmax), kk(ixmax)
      common /pk_data/ ak, pk, ik_max
      common /halo_pk_data/ ak_nl, pk_nl, rknl, ik_max_nl 
      common /alpha_data/ eta, alphaz, iz_max
      pi = 4.d0 * datan(1.d0)
c     -------------------------------------------------
c
      zmax = zred + 10.d0
c
      do iz=1,  izmax
c
         z = zred + (zmax-zred)*dble(iz-1)/dble(izmax-1)
         eta(iz) = dlog(growth(z))
c
         call calc_halo_pk(z)
c
c     ////////  Compute alpha^2(z) correction  ////////  
c
         denom=0.d0
         numer=0.d0
         kmin = max( ak(1), ak_nl(1) )
c
         if(ibox.eq.0) kmin = 2.d0 * pi / Lbox
c
         kmax = min( ak(ik_max), ak_nl(ik_max_nl) ) 
c
         call gauleg(dlog(kmin), dlog(min(1.d0*rknl, kmax)), kk, 
     &        wk, ixmax)
c
         do ix=1, ixmax
            kk(ix) = dexp(kk(ix))
            call find_pk(kk(ix), pklin)
            call find_pknl(kk(ix), pknl)
            numer = numer + wk(ix) * pknl * kk(ix)
            denom = denom + wk(ix) * pklin  * kk(ix)
         enddo
c
         alphaz(iz) = dsqrt(numer / denom) / growth(z)
c
         write(6,'(1p4e18.10)') z, growth(z), rknl, alphaz(iz)
c
         if( alphaz(iz).le.1.d0 ) goto 10
c
      enddo
c
 10   iz_max = iz - 1 
c
      end
c
c ******************************************************* c
c
      subroutine find_alpha(eta1, alpha_z)
c
c ******************************************************* c
c
      implicit none
      integer izmax, iz_max
      parameter(izmax=100)
      integer j, jmin, jmax
      real*8  eta1, eta(izmax), alphaz(izmax)
      real*8  s, ds, alpha_z
      common /alpha_data/ eta, alphaz, iz_max
c     -------------------------------------------
c
      if(eta1.le.eta(1) .and. eta1.ge.eta(iz_max)) then
c
         call hunt(eta, iz_max, eta1, j)
c
         jmin = j - 2
         jmax = j + 2
         if(jmin.lt.1) jmin = 1
         if(jmax.ge.iz_max) jmax = iz_max
c
         call polint(eta(jmin),alphaz(jmin),jmax-jmin+1,eta1,s,ds)
         alpha_z = s
c
      else
         alpha_z = 1.d0
      endif
c      
      end
c
c ******************************************************* c
c
      subroutine find_pknl(kk, pknl)
c
c ******************************************************* c
c
      implicit none
      integer ik_max, ikmax
      integer j, jmin, jmax
      parameter(ikmax=5000)
      real*8 ak_nl(ikmax), pk_nl(ikmax), kk, s, ds, pknl, rknl
      common /halo_pk_data/ ak_nl, pk_nl, rknl, ik_max
c     -------------------------------------------
c
      call hunt(ak_nl, ik_max, kk, j)
c
      jmin = j - 2
      jmax = j + 2
      if(jmin.lt.1) jmin = 1
      if(jmax.ge.ik_max) jmax = ik_max
c
      call polint(ak_nl(jmin),pk_nl(jmin),jmax-jmin+1,kk,s,ds)
      pknl = s
c      
      end
c
c ***************************************************** c
c
      subroutine calc_halo_pk(zred)
c
c ***************************************************** c
c
      implicit none
c
      integer ikmax, ik_max_nl, ik_max, ik, icount
      parameter(ikmax=5000)
      real*8 ak(ikmax), pk(ikmax)
      real*8 ak_nl(ikmax), pk_nl(ikmax)
      real*8 zred, pi, growth
c
      real*8 xlogr1, xlogr2, rmid, sig, d1, d2
      real*8 diff, rknl, rneff, rncur, Del_lin, Del_nl
      real*8 pq, ph
c
      common /halo_pk_data/ ak_nl, pk_nl, rknl, ik_max_nl
      common /pk_data/ ak, pk, ik_max
      pi = 4.d0 * datan(1.d0)
c     -----------------------------------------  c
c
c calculate nonlinear wavenumber (rknl), effective spectral index (rneff) and 
c curvature (rncur) of the power spectrum at the desired redshift, using method 
c described in Smith et al (2002).
c
      icount = 0
ccc      xlogr1=-2.0    A.Taruya 2008/12/03
      xlogr1=-3.0
      xlogr2=3.5
 10   rmid=(xlogr2+xlogr1)/2.0
      rmid=10**rmid
c
cccc  A.Taruya 2008/12/03  force to stop iteration
      icount = icount + 1
      if(icount.gt.30) then
cccc         write(6,*) ' computation of rknl fails to converge '
         rknl=1./rmid
         rneff=-3-d1
         rncur=-d2
         goto 20
      endif
cccc  A.Taruya 2008/12/03 
c
      call wint(rmid,sig,d1,d2,zred)
      diff=sig-1.0
      if (abs(diff).le.0.001) then
         rknl=1./rmid
         rneff=-3-d1
         rncur=-d2                  
      elseif (diff.gt.0.001) then
         xlogr1=log10(rmid)
         goto 10
      elseif (diff.lt.-0.001) then
         xlogr2=log10(rmid)
         goto 10
      endif
c
ccc      write(*,20) 'rknl [h/Mpc] =',rknl,'rneff=',rneff, 'rncur=',rncur
ccc 20   format(a14,f12.6,2x,a6,f12.6,2x,a6,f12.6)
c
ccc      pause
c
 20   ik_max_nl = ik_max
c
      do ik=1, ik_max_nl
c
c linear power spectrum !! Remeber => plin = k^3 * P(k) * constant
c constant = 4*pi*V/(2*pi)^3 
c
         Del_lin = growth(zred)**2 * ak(ik)**3 * pk(ik) 
     &        / (2.d0*pi*pi)
c
c calculate nonlinear power according to halofit: pnl = pq + ph,
c where pq represents the quasi-linear (halo-halo) power and 
c where ph is represents the self-correlation halo term. 
c 
         call halofit(zred,ak(ik),rneff,rncur,rknl,
     &        Del_lin,Del_nl,pq,ph)
c
         ak_nl(ik) = ak(ik)
         pk_nl(ik) = Del_nl / Del_lin * pk(ik) * growth(zred)**2
ccc         write(6,*) ak_nl(ik), pk_nl(ik)
c
      enddo
c
      end
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

c The subroutine wint, finds the effective spectral quantities
c rknl, rneff & rncur. This it does by calculating the radius of 
c the Gaussian filter at which the variance is unity = rknl.
c rneff is defined as the first derivative of the variance, calculated 
c at the nonlinear wavenumber and similarly the rncur is the second
c derivative at the nonlinear wavenumber. 

      subroutine wint(r,sig,d1,d2,zred)
      
      implicit none
      real*8 sum1,sum2,sum3,t,y,x,w1,w2,w3
      real*8 rk, pklin, r, sig, d1,d2
      real*8 growth, zred, pi
      integer i,nint
      pi = 4.d0 * datan(1.d0)
c
      nint=3000
      sum1=0.d0
      sum2=0.d0
      sum3=0.d0
      do i=1,nint
         t=(float(i)-0.5)/float(nint)
         y=-1.d0+1.d0/t
         rk=y
         call find_pk(rk, pklin)
         d2= growth(zred)**2 * pklin * rk**3 / (2.d0*pi*pi)
         x=y*r
         w1=exp(-x*x)
         w2=2*x*x*w1
         w3=4*x*x*(1-x*x)*w1
         sum1=sum1+w1*d2/y/t/t
         sum2=sum2+w2*d2/y/t/t
         sum3=sum3+w3*d2/y/t/t
      enddo
      sum1=sum1/float(nint)
      sum2=sum2/float(nint)
      sum3=sum3/float(nint)
      sig=sqrt(sum1)
      d1=-sum2/sum1
      d2=-sum2*sum2/sum1/sum1 - sum3/sum1
      
      return
      end
c      
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c halo model nonlinear fitting formula as described in 
c Appendix C of Smith et al. (2002)
c
      subroutine halofit(zred,rk,rn,rncur,rknl,plin,pnl,pq,ph)
c
      implicit none
c
      real*8 gam,a,amod,b,c,xmu,xnu,alpha,beta,f1,f2,f3,f4
      real*8 rk,rn,plin,pnl,pq,ph
      real*8 om_m,om_v,omg_m,omg_v,aexp
      real*8 Omega_m, Omega_v, w_de, sigma8, zred
      real*8 rknl,y,rncur,p_cdm
      real*8 f1a,f2a,f3a,f4a,f1b,f2b,f3b,f4b,frac
      common /cosmological_param/ Omega_m, Omega_v, w_de, sigma8
c     --------------------------------------------  c
c
      aexp = 1.d0 / (1.d0 + zred)
      om_m = omg_m(aexp,Omega_m,Omega_v,w_de) 
      om_v = omg_v(aexp,Omega_m,Omega_v,w_de) 
c
      gam=0.86485+0.2989*rn+0.1631*rncur
      a=1.4861+1.83693*rn+1.67618*rn*rn+0.7940*rn*rn*rn+
     &0.1670756*rn*rn*rn*rn-0.620695*rncur
      a=10**a      
      b=10**(0.9463+0.9466*rn+0.3084*rn*rn-0.940*rncur)
      c=10**(-0.2807+0.6669*rn+0.3214*rn*rn-0.0793*rncur)
      xmu=10**(-3.54419+0.19086*rn)
      xnu=10**(0.95897+1.2857*rn)
      alpha=1.38848+0.3701*rn-0.1452*rn*rn
      beta=0.8291+0.9854*rn+0.3400*rn**2
c
      if(abs(1-om_m).gt.0.01) then ! omega evolution 
         f1a=om_m**(-0.0732)
         f2a=om_m**(-0.1423)
         f3a=om_m**(0.0725)
         f1b=om_m**(-0.0307)
         f2b=om_m**(-0.0585)
         f3b=om_m**(0.0743)       
         frac=om_v/(1.-om_m) 
         f1=frac*f1b + (1-frac)*f1a
         f2=frac*f2b + (1-frac)*f2a
         f3=frac*f3b + (1-frac)*f3a
      else         
         f1=1.0
         f2=1.
         f3=1.
      endif
c
      y=(rk/rknl)
c
      ph=a*y**(f1*3)/(1+b*y**(f2)+(f3*c*y)**(3-gam))
      ph=ph/(1+xmu*y**(-1)+xnu*y**(-2))
      pq=plin*(1+plin)**beta/(1+plin*alpha)*exp(-y/4.0-y**2/8.0)
c
      pnl=pq+ph
c
      return
      end     
c 
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c evolution of omega matter with expansion factor
c
cc      function omg_m(aa,om_m0,om_v0)
cc      implicit none
cc      real*8 omg_m,omega_t,om_m0,om_v0,aa
cc      omega_t=1.0+(om_m0+om_v0-1.0)/(1-om_m0-om_v0+om_v0*aa*aa+om_m0/aa)
cc      omg_m=omega_t*om_m0/(om_m0+om_v0*aa*aa*aa)
cc      return
cc      end
c
      function omg_m(aa,om_m0,om_v0,w_de)
      implicit none
      real*8 omg_m,omega_k,om_m0,om_v0,w_de,aa
      omega_k = om_m0 + om_v0 - 1.d0 
      omg_m = om_m0 / ( om_m0 + om_v0/aa**(3.0*w_de) - omega_k*aa )
      return
      end
c
c%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
c
c evolution of omega lambda with expansion factor
c
cc      function omg_v(aa,om_m0,om_v0)      
cc      implicit none
cc      real*8 aa,omg_v,om_m0,om_v0,omega_t
cc      omega_t=1.0+(om_m0+om_v0-1.0)/(1-om_m0-om_v0+om_v0*aa*aa+om_m0/aa)
cc      omg_v=omega_t*om_v0/(om_v0+om_m0/aa/aa/aa)
cc      return
cc      end
c
      function omg_v(aa,om_m0,om_v0,w_de)      
      implicit none
      real*8 aa,omg_v,om_m0,om_v0,w_de,omega_k
      omega_k = om_m0 + om_v0 - 1.d0 
      omg_v = om_v0/aa**(3.0*w_de) 
     &     / ( om_m0 + om_v0/aa**(3.0*w_de) - omega_k*aa )
      return
      end
