c ******************************************************* c
c
      subroutine calc_renormalized_pk(itype, ijob, zred)
c                                                         c
c                                                         c
c             Time-stamp: <2012-07-31 15:46:06 ataruya>   c
c ******************************************************* c
c
c     Computing 2nd-order Born corrections of improved PT power spectra
c     using hybrid integration scheme (Gaussian quadrature for time 
c     integral (2D), Monte Calro method for Fourier integral (4D))
c
c     This code should be compiled with CUBA library, libcuba.a
c
      implicit none
c
      integer ndim4, ncomp, mineval, maxeval
      integer verbose, last
      double precision epsrel, epsabs
      parameter (ndim4 = 4)
      parameter (ncomp = 1)
      parameter (epsrel = 1D-3)
      parameter (epsabs = 1D-12)
      parameter (verbose = 0)
      parameter (last = 4)
      parameter (mineval = 0)
c     for Suave 
      parameter (maxeval=50000)
c     for Cuhre
c      parameter (maxeval=100000)
      integer nnew
      double precision flatness
      parameter (nnew=1000)
      parameter (flatness=15D0)
      integer key
      parameter (key = 0)
      double precision integral(ncomp), error(ncomp), prob(ncomp)
      integer nregions, neval, fail
c     ---------------------------------------------------
      integer itype, iitype, ijob 
      integer ik, ik_init, ikmax, ik_max1
      integer ix, ixmax
      parameter(ikmax=5000)
      real*8  ak1(ikmax), pk(ikmax), pk_2loop(ikmax) 
      real*8  zred, zzred, growth, pi
      real*8  k, kmin, kmax, k_cut, k_cut2
      external integ_pk_2loop
      common /pk_data/ ak1, pk, ik_max1
      common /wave_number/  k, zzred, kmin, kmax, iitype
c     -------------------------------------------------
      pi = 4.d0 * datan(1.d0)
      iitype = itype
      zzred = zred
c     -------------------------------------------------
c
      write(6,*) ' input k_min for output data'
      read(5,*) k_cut
      write(6,*) ' input k_max for output data'
      read(5,*) k_cut2
c
      if(ijob.eq.0) then
         call search(ik_init)
         open(10,file='renormalized_pkreal_2ndBorn.dat',
     &        status='unknown', access='append')
      elseif(ijob.eq.1) then 
         ik_init = 1
         open(10,file='renormalized_pkreal_2ndBorn.dat',
     &        status='unknown', access='sequential')
      endif
c
c     //////// calculation of 2-loop P(k) ////////c
c
      kmin = ak1(1)
      kmax = ak1(ik_max1)
c
      do 10 ik=ik_init, ik_max1
c
         k = ak1(ik)
c
         pk_2loop(ik) = 0.0d0
c
c     skip the integration for small-k or large-k mode
c
         if(ak1(ik).lt.k_cut) goto 9
         if(ak1(ik).gt.k_cut2) goto 100
c
c     integration by Monte Carlo method
c
         call suave(ndim4, ncomp, integ_pk_2loop, 
     &        epsrel, epsabs, verbose + last, mineval, maxeval,
     &        nnew, flatness, 
     &        nregions, neval, fail, integral, error, prob)
ccc         call cuhre(ndim4, ncomp, integ_pk_2loop, 
ccc     &        epsrel, epsabs, verbose + last, mineval, maxeval,
ccc     &        key,
ccc     &        nregions, neval, fail, integral, error, prob)
c
         pk_2loop(ik) = integral(1) * k**6 / (2.d0*pi)**4 * 4.d0
c
         if(itype.eq.0) pk_2loop(ik)=16.d0 * pk_2loop(ik)
         if(itype.eq.1) pk_2loop(ik)= 8.d0 * pk_2loop(ik)
c
ccc         write(6,'(i4, 1p2e18.10)') ik, k, pk_2loop(ik)
c
 9    continue
c
      write(10,'(1p4e18.10)') 
     &     ak1(ik), growth(zred)**2*pk(ik), pk_2loop(ik)
c
 10   continue
c
 100  continue
c
      close(10)
c     
      end
c
c ******************************************************* c
c
      subroutine integ_pk_2loop(ndim, xx, ncomp, ff)
c
c ******************************************************* c
c
c     integrand of 2-loop correction
c
      implicit none
c
      integer ndim, ncomp
      double precision xx(*), ff(*)
      double precision pi
      parameter (pi = 3.14159265358979323846D0)
c --------------------------------------- c      
      integer itype
      real*8  zred, zinit
      real*8  integ_pk_2loop_A, integ_pk_2loop_B
      parameter(zinit=500.d0)
      real*8  kmin, kmax, jacobian
      real*8  k, x, mu, y, nu, func_J1, func_K1, growth
      real*8  kp, q, pq, p, kpq
      real*8  pklin_p, pklin_q, pklin_kp, pklin_pq, pklin_kpq
      common /wave_number/  k, zred, kmin, kmax, itype
c
c
      x = dexp( dlog(kmin/kmax) + 
     &     ( dlog(kmax/k+1.d0) - dlog(kmin/kmax) ) * xx(1) )
      mu = -1.d0 + 2.d0 * xx(2)
      y = dexp( dlog(kmin/kmax) + 
     &     ( dlog(kmax/kmin) - dlog(kmin/kmax) ) * xx(3)  )
      nu = -1.d0 + 2.d0 * xx(4)
      jacobian = ( dlog(kmax/k+1.d0) - dlog(kmin/kmax) ) 
     &     * ( dlog(kmax/kmin) - dlog(kmin/kmax) ) * 4.d0 
c
c
      if( (1.d0 + x*x - 2.d0*x*mu .ge.0.d0).and.
     &     (1.d0 + y*y - 2.d0*y*nu .ge.0.d0) 
     &     .and. (x.lt.0.5d0 .or. 
     &     (x.ge.0.5d0 .and. (mu.lt.0.5d0/x)) ) ) then 
c
c ///// A-part ///// c
c
         kp = k * dsqrt(1.d0 + x*x - 2.d0*x*mu)
         q =  k * x * y 
         pq = k * x * dsqrt(1.d0 + y*y - 2.d0*y*nu)  
c
         if( ((kp.ge.kmin).and.(kp.le.kmax)) .and. 
     &        ((q.ge.kmin).and.(q.le.kmax)) .and. 
     &        ((pq.ge.kmin).and.(pq.le.kmax)) ) then 
            if( y.lt.0.5d0 .or. 
     &           (y.ge.0.5d0 .and. (nu.lt.0.5d0/y)) ) then
               integ_pk_2loop_A = x**5 * y**2 * 
     &              func_J1(itype, k, x, y, mu, nu, zred, zinit)**2 
               call find_pk(k*x*y, pklin_q)
               call find_pk(k*dsqrt(1.d0+x*x-2.d0*mu*x), pklin_kp)
               call find_pk(k*x*dsqrt(1.d0+y*y-2.d0*nu*y), pklin_pq)
c
               integ_pk_2loop_A = integ_pk_2loop_A * 
     &              pklin_q * pklin_kp * pklin_pq * growth(zinit)**6
            else
               integ_pk_2loop_A = 0.d0
            endif
         else
            integ_pk_2loop_A = 0.d0
         endif
c
c ///// B-part ///// c
c
         p = k * x
         q = k * y * dsqrt(1.d0 + x*x - 2.d0*x*mu)  
         kpq = k * dsqrt(1.d0 + x*x - 2.d0*x*mu) 
     &        *  dsqrt(1.d0 + y*y - 2.d0*y*nu) 
c
         if( ((p.ge.kmin).and.(p.le.kmax)) .and. 
     &        ((q.ge.kmin).and.(q.le.kmax)) .and. 
     &        ((kpq.ge.kmin).and.(kpq.le.kmax)) ) then 
            if( y.lt.0.5d0 .or. 
     &           (y.ge.0.5d0 .and. (nu.lt.0.5d0/y)) ) then
               integ_pk_2loop_B = x**2 * y**2 
     &              * (1.d0+x*x-2.d0*mu*x)**1.5
     &              * func_K1(itype, k, x, y, mu, nu, zred, zinit)**2
               call find_pk(k*dsqrt(1.d0+x*x-2.d0*mu*x)*y, pklin_q)
               call find_pk(k*x, pklin_p)
               call find_pk(k*dsqrt(1.d0+x*x-2.d0*mu*x)*
     &              dsqrt(1.d0+y*y-2.d0*nu*y), pklin_kpq)
c     
               integ_pk_2loop_B = integ_pk_2loop_B * 
     &              pklin_q * pklin_p * pklin_kpq * growth(zinit)**6
            else
               integ_pk_2loop_B = 0.d0
            endif
         else
            integ_pk_2loop_B = 0.d0
         endif
      else
         integ_pk_2loop_A = 0.0d0
         integ_pk_2loop_B = 0.0d0
      endif
c
ccccccc !!! check !!!! ccccccc
c
ccc      if(integ_pk_2loop_A.gt.0.d0 .and. integ_pk_2loop_B.gt.0.d0) then
ccc         if(k*x.ge.5.e-2  .and. k*dsqrt(1.d0+x*x-2.d0*mu*x)*y.ge.5.e-2) 
ccc     &        write(6,'(1p7e12.4)') k, x, y, mu, nu, 
ccc     &        integ_pk_2loop_A, integ_pk_2loop_B
ccc      endif
c
ccccccc !!! check !!!! ccccccc
c
      ff(1)= (integ_pk_2loop_A + integ_pk_2loop_B) * x * y / 2.d0
c
      ff(1) = ff(1) * jacobian
c
      end
c
c ******************************************************* c
c
      function func_J1(itype, k, x, y, mu, nu, zred, zinit)
c
c ******************************************************* c
c
c     computing function J_1 using method of Gaussian quadratures 
c
      implicit none
      integer itype, iitype
      integer ieta, ieta_max
      parameter(ieta_max=40)
      real*8  k, x, y, mu, nu, zred, zinit, Lambda
      real*8  func_J1, int_J1, func_J1a
      real*8  k1, k2, k3, k4, k5, eta0, eta, ss
      real*8  kk, xx, yy, mmu, nnu
      real*8  ff, gg, hh, ii, jj
      real*8  growth, weta(ieta_max), xeta(ieta_max)
      common /integ_J1/ kk, xx, yy, mmu, nnu, eta0, eta, iitype 
c     ------------------------------------------
c
      k1 = k
      k2 = k * x
      k3 = k * dsqrt(1.d0 - 2.d0*x*mu + x*x)
      k4 = k * x * y
      k5 = k * x * dsqrt(1.d0 - 2.d0*y*nu + y*y)
c
      eta0 = dlog(growth(zinit))
      eta  = dlog(growth(zred))
c
      if(max(k1,k2,k3,k4,k5).le.5.d-3) then 
c
c     using the linear propagator, integration is analytically performed
c
         ss = eta - eta0
c
         ff = 28.d0*x*x*y*mu * ( nu*nu - 1.d0 ) 
     &        - 9.d0 * mu * ( y - 7.d0*nu + 6.d0*y*nu*nu ) 
     &        + 7.d0 *x* ( 5.d0 - 14.d0*mu*mu ) * nu
     &        + x*y * ( 23.d0 - 58.d0*nu*nu 
     &        + 14.d0*mu*mu * ( 1.d0 + 6.d0*nu*nu ) )
         ff = ff / 504.d0
         gg = ( -7.d0*mu + x*(-3.d0 + 10.d0*mu*mu ) ) 
     &        * ( -5.d0*nu + y*(-1.d0 + 6.d0*nu*nu ) ) 
         gg = -gg / 140.d0
         hh = -4.d0*x*x*y*mu * ( nu*nu - 1.d0 ) 
     &        + 5.d0*mu * ( y + nu - 2.d0*y*nu*nu ) 
     &        + x * ( 1.d0 - 6.d0*mu*mu ) * nu
     &        + x*y * ( -3.d0 + 2.d0*nu*nu 
     &        + 6.d0*mu*mu * ( 2.d0*nu*nu - 1.d0 ) )
         hh = hh / 40.d0
         ii = -2.d0/35.d0 * ( x - mu ) * ( nu*nu -1.d0 ) * y
         jj = -2.d0*( mu*mu - 1.d0 )*nu - y*( nu*nu + 1.d0 ) 
     &        - y*mu*( 7.d0*x - 8.d0*mu + (-7.d0*x + 6.d0*mu)*nu*nu ) 
         jj = 2.d0/315.d0 * jj * x
c
         func_J1 = ff * dexp(3.d0*ss) + gg * dexp(2.d0*ss) + hh * 
     &        dexp(ss) + ii * dexp(-0.5d0*ss) +jj * dexp(-1.5d0*ss)
         func_J1 = func_J1 / ( 1.d0 + x*x - 2.d0*x*mu ) 
     &        / ( 1.d0 + y*y - 2.d0*y*nu ) / x / y 
c
ccc         func_J1a = ff * dexp(3.d0*ss) + gg * dexp(2.d0*ss) + hh * 
ccc     &        dexp(ss) + ii * dexp(-0.5d0*ss) +jj * dexp(-1.5d0*ss)
ccc         func_J1a = func_J1a / ( 1.d0 + x*x - 2.d0*x*mu ) 
ccc     &        / ( 1.d0 + y*y - 2.d0*y*nu ) / x / y 
c
      elseif(max(k1,k2,k3,k4,k5).gt.5.d-3) then 
c
c     numerical integration by Gaussian quadratures
c
         kk = k
         xx = x
         yy = y
         mmu = mu
         nnu = nu
         iitype = itype
c
         func_J1 = 0.0d0
c
         call gauleg(eta0, eta, xeta, weta, ieta_max)
c
         do ieta=1, ieta_max
            func_J1 = func_J1 + weta(ieta) * int_J1(xeta(ieta))
         enddo
c
ccccccc !!! check !!!! ccccccc
c
ccc         if(max(k1,k2,k3,k4,k5).le.1.d-2) 
ccc     &        write(6,*) 'J1=',func_J1a, func_J1
c
ccccccc !!! check !!!! ccccccc
c
      endif
c
      end
c
c ******************************************************* c
c
      function int_J1(eta1)
c
c ******************************************************* c
c
      implicit none
      integer itype
      real*8  int_J1, eta1, eta, eta0
      real*8  kxmu, k, x, y, mu, nu
      real*8  G11a, G12a, G21a, G22a, G11c, G12c, G21c, G22c
      real*8  I1, I2
      real*8  gamma112, gamma121, gamma222, Lambda
      common /integ_J1/ k, x, y, mu, nu, eta0, eta, itype 
c     ---------------------------------
c
      kxmu = k * dsqrt(1.d0 - 2.d0*x*mu + x*x)
c
      if(itype.eq.0) then 
         call G_ab_RPT(k, eta, eta1, G11a, G12a, G21a, G22a)
         call G_ab_RPT(kxmu, eta1, eta0, G11c, G12c, G21c, G22c)
      elseif(itype.eq.1) then
         call G_ab_Closure(k, eta, eta1, G11a, G12a, G21a, G22a)
         call G_ab_Closure(kxmu, eta1, eta0, G11c, G12c, G21c, G22c)
      endif
c
      gamma112 = 0.5d0 * (1.d0 - x * mu) / (1.d0 - 2.d0*x*mu + x*x) 
      gamma121 = 0.5d0 * mu / x
      gamma222 = 0.5d0 * ( mu - x ) / (1.d0 - 2.d0*x*mu + x*x) / x
c
      I1 = Lambda(itype, 1, k*x, y, nu, eta1, eta0)
      I2 = Lambda(itype, 2, k*x, y, nu, eta1, eta0)
c
      int_J1 = G11a * ( gamma112 * I1 * ( G21c + G22c )
     &     + gamma121 * I2 * ( G11c + G12c ) )
     &     + G12a * gamma222 * I2 * ( G21c + G22c )
c
      end
c
c ******************************************************* c
c
      function func_K1(itype, k, x, y, mu, nu, zred, zinit)
c
c ******************************************************* c
c
c     computing function K_1 using method of Gaussian quadratures 
c
      implicit none
      integer itype, iitype
      integer ieta, ieta_max
      parameter(ieta_max=40)
      real*8  k, x, y, mu, nu, zred, zinit, Lambda
      real*8  func_K1, int_K1, func_K1a
      real*8  k1, k2, k3, k4, k5, eta0, eta, ss
      real*8  kk, xx, yy, mmu, nnu
      real*8  ff, gg, hh, ii, jj
      real*8  growth, weta(ieta_max), xeta(ieta_max)
      common /integ_K1/ kk, xx, yy, mmu, nnu, eta0, eta, iitype 
c     ------------------------------------------
c
      k1 = k
      k2 = k * x
      k3 = k * dsqrt(1.d0 - 2.d0*x*mu + x*x)
      k4 = k * dsqrt(1.d0 - 2.d0*x*mu + x*x) * y
      k5 = k * dsqrt(1.d0 - 2.d0*x*mu + x*x) 
     &     * dsqrt(1.d0 - 2.d0*y*nu + y*y)
c
      eta0 = dlog(growth(zinit))
      eta  = dlog(growth(zred))
c
      if(max(k1,k2,k3,k4,k5).le.5.d-3) then 
c
c     using the linear propagator, integration is analytically performed
c
         ss = eta - eta0
c
         ff = -28.d0*x*x*y*mu * ( nu*nu - 1.d0 ) 
     &        + mu * ( 19.d0*y + 63.d0*nu - 82.d0*y*nu*nu ) 
     &        + 7.d0 *x* ( 5.d0 - 14.d0*mu*mu ) * nu
     &        + x*y * ( -5.d0 - 30.d0*nu*nu 
     &        + 14.d0*mu*mu * ( 10.d0*nu*nu - 3.d0 ) )
         ff = ff / 504.d0
         gg = ( -7.d0*mu + x*(-3.d0 + 10.d0*mu*mu ) ) 
     &        * ( -5.d0*nu + y*(-1.d0 + 6.d0*nu*nu ) ) 
         gg = -gg / 140.d0
         hh = 4.d0*x*x*y*mu * ( nu*nu - 1.d0 ) 
     &        + mu * ( y + 5.d0*nu - 6.d0*y*nu*nu ) 
     &        + x * ( 1.d0 - 6.d0*mu*mu ) * nu
     &        + x*y * ( 1.d0 - 2.d0*nu*nu 
     &        + 2.d0*mu*mu * ( 2.d0*nu*nu + 1.d0 ) )
         hh = hh / 40.d0
         ii = -2.d0/35.d0 * ( x - mu ) * ( nu*nu -1.d0 ) * y
         jj = 7.d0*y*mu * ( nu*nu-1.d0 ) + 7.d0*x*x*y*mu * 
     &        ( nu*nu -1.d0 ) - 2.d0*x* ( 1.d0 - mu*mu ) * nu 
     &        - 2.d0*x*y * ( -4.d0 + 3.d0*nu*nu + mu*mu * 
     &        ( -3.d0 + 4.d0*nu*nu ) )
         jj = -2.d0/315.d0 * jj
c
         func_K1 = ff * dexp(3.d0*ss) + gg * dexp(2.d0*ss) + hh * 
     &        dexp(ss) + ii * dexp(-0.5d0*ss) +jj * dexp(-1.5d0*ss)
         func_K1 = func_K1 / ( 1.d0 + x*x - 2.d0*x*mu ) 
     &        / ( 1.d0 + y*y - 2.d0*y*nu ) / x / y 
c
ccc         func_K1a = ff * dexp(3.d0*ss) + gg * dexp(2.d0*ss) + hh * 
ccc     &        dexp(ss) + ii * dexp(-0.5d0*ss) +jj * dexp(-1.5d0*ss)
ccc         func_K1a = func_K1a / ( 1.d0 + x*x - 2.d0*x*mu ) 
ccc     &        / ( 1.d0 + y*y - 2.d0*y*nu ) / x / y 
c
      elseif(max(k1,k2,k3,k4,k5).gt.5.d-3) then 
c
c     numerical integration by Gaussian quadratures
c
         kk = k
         xx = x
         yy = y
         mmu = mu
         nnu = nu
         iitype = itype
c
         func_K1 = 0.0d0
c
         call gauleg(eta0, eta, xeta, weta, ieta_max)
c
         do ieta=1, ieta_max
            func_K1 = func_K1 + weta(ieta) * int_K1(xeta(ieta))
         enddo
c
ccccccc !!! check !!!! ccccccc
c
ccc         if(max(k1,k2,k3,k4,k5).le.1.d-2) 
ccc     &        write(6,*) 'K1=',func_K1a, func_K1
c
ccccccc !!! check !!!! ccccccc
c
      endif
c
      end
c
c ******************************************************* c
c
      function int_K1(eta1)
c
c ******************************************************* c
c
      implicit none
      integer itype
      real*8  int_K1, eta1, eta, eta0
      real*8  k, x, y, mu, nu
      real*8  G11a, G12a, G21a, G22a, G11c, G12c, G21c, G22c
      real*8  I1, I2
      real*8  gamma112, gamma121, gamma222, Lambda
      common /integ_K1/ k, x, y, mu, nu, eta0, eta, itype 
c     ---------------------------------
c
c
      if(itype.eq.0) then 
         call G_ab_RPT(k, eta, eta1, G11a, G12a, G21a, G22a)
         call G_ab_RPT(k*x, eta1, eta0, G11c, G12c, G21c, G22c)
      elseif(itype.eq.1) then
         call G_ab_Closure(k, eta, eta1, G11a, G12a, G21a, G22a)
         call G_ab_Closure(k*x, eta1, eta0, G11c, G12c, G21c, G22c)
      endif
c
      gamma112 = 0.5d0 * (1.d0 - x * mu) / (1.d0 - 2.d0*x*mu + x*x) 
      gamma121 = 0.5d0 * mu / x
      gamma222 = 0.5d0 * ( mu - x ) / (1.d0 - 2.d0*x*mu + x*x) / x
c
      I1 = Lambda(itype, 1, k*dsqrt(1.d0+x*x-2.d0*x*mu), 
     &     y, nu, eta1, eta0)
      I2 = Lambda(itype, 2, k*dsqrt(1.d0+x*x-2.d0*x*mu), 
     &     y, nu, eta1, eta0)
c
      int_K1 = G11a * ( gamma112 * ( G11c + G12c ) * I2 
     &     + gamma121 * ( G21c + G22c ) * I1 )
     &     + G12a * gamma222 * ( G21c + G22c ) * I2 
c
      end
c
c ******************************************************* c
c
      function  Lambda(itype, isub, k, x, mu, eta1, eta0)
c
c ******************************************************* c
c
c     computing the function Lambda by Gaussian quadratures
c
c     itype : RPT or  CLA
c     isub  : Lamda_1 or Lambda_2 (label of subscript)
c
      implicit none
c
      integer isub, itype, iitype
      integer ieta, ieta_max
      parameter(ieta_max=40)
      real*8  k, k1, k2, k3, zred, x, mu, Lambda
      real*8  kk, zredinit, zzred, xx, mmu, Lambda_s
      real*8  zinit
      real*8  ss, eta0, eta1, eeta0, eeta1
      real*8  int_lambda
      real*8  eta_s1, eta_s2, Lambda_trapzd
      real*8  weta(ieta_max), xeta(ieta_max)
      common /integ_lambda/ kk, xx, mmu, eeta0, eeta1, iitype 
c     ---------------------------------------
c
      k1 = k
      k2 = k * x
      k3 = k * dsqrt(1.d0 - 2.d0*x*mu + x*x)
c
      if(max(k1,k2,k3).le.5.d-3) then 
c
c     using the linear propagator, integration is analytically performed
c
         ss = eta1 - eta0 
c
         if(isub.eq.1) then
            Lambda = 4.d0/35.d0 * x * (-1.d0 + mu*mu) * dexp(-1.5d0*ss)
     &           - 0.1d0 * (x + 5.d0*mu - 6.d0*x*mu*mu) * dexp(ss) 
     &           + 1.d0/14.d0 * (7.d0*mu + x * (3.d0 - 10.d0*mu*mu)) 
     &           * dexp(2.d0*ss)
            Lambda = Lambda / x / (1.d0 + x*x -2.d0*x*mu) 
         elseif(isub.eq.2) then
            Lambda = -6.d0/35.d0 * x * (-1.d0 + mu*mu) * dexp(-1.5d0*ss)
     &           - 0.1d0 * (x + 5.d0*mu - 6.d0*x*mu*mu) * dexp(ss) 
     &           + 1.d0/14.d0 * (7.d0*mu - x  - 6.d0*x*mu*mu) 
     &           * dexp(2.d0*ss)
            Lambda = Lambda / x / (1.d0 + x*x -2.d0*x*mu) 
         endif
c
      elseif(max(k1,k2,k3).gt.5.d-3) then 
c
         iitype = itype
         kk = k
         xx = x
         mmu = mu
         eeta0 = eta0
         eeta1 = eta1
c
c     integral over "eta" by Gaussian quadratures 
c     
         Lambda = 0.d0
c
         call gauleg(eta0, eta1, xeta, weta, ieta_max)
c
         do ieta=1, ieta_max
            Lambda = Lambda + weta(ieta) * int_lambda(isub,xeta(ieta))
         enddo
c
      endif
c
      end
c
c ******************************************************* c
c
      function int_lambda(isub, eta2)
c
c ******************************************************* c
c
c     integrand of function Lambda, as function of eta_s
c
      implicit none
c
      integer isub, itype
      real*8 int_lambda, eta0, eta1, eta2 
      real*8 k, x, mu
      real*8 k1, k2, k3
      real*8 G11a, G12a, G21a, G22a
      real*8 G11b, G12b, G21b, G22b
      real*8 G11c, G12c, G21c, G22c
      real*8 gamma112, gamma121, gamma222
      common /integ_lambda/ k, x, mu, eta0, eta1, itype
c     ------------------------------------------
c
      k1 = k
      k2 = k * x
      k3 = k * dsqrt( 1.d0 - 2.d0*x*mu + x*x )
c
      if(itype.eq.0) then 
         call G_ab_RPT(k1, eta1, eta2, G11a, G12a, G21a, G22a)
         call G_ab_RPT(k2, eta2, eta0, G11b, G12b, G21b, G22b)
         call G_ab_RPT(k3, eta2, eta0, G11c, G12c, G21c, G22c)
      elseif(itype.eq.1) then
         call G_ab_Closure(k1, eta1, eta2, G11a, G12a, G21a, G22a)
         call G_ab_Closure(k2, eta2, eta0, G11b, G12b, G21b, G22b)
         call G_ab_Closure(k3, eta2, eta0, G11c, G12c, G21c, G22c)
      endif
c
      gamma112 = 0.5d0 * (1.d0 - x * mu) / (1.d0 - 2.d0*x*mu + x*x) 
      gamma121 = 0.5d0 * mu / x
      gamma222 = 0.5d0 * ( mu - x ) / (1.d0 - 2.d0*x*mu + x*x) / x
c
      if(isub.eq.1) 
     &     int_lambda = G11a * ( gamma112 * (G11b + G12b)*(G21c + G22c)
     &     + gamma121 * (G21b + G22b) * (G11c + G12c) )
     &     + G12a * gamma222 * (G21b + G22b) * (G21c + G22c) 
      if(isub.eq.2)
     &     int_lambda = G21a * ( gamma112 * (G11b + G12b)*(G21c + G22c)
     &     + gamma121 * (G21b + G22b) * (G11c + G12c) )
     &     + G22a * gamma222 * (G21b + G22b) * (G21c + G22c) 
c
      end
c
c ******************************************************* c
c
      subroutine search(ik_init)
c
c ******************************************************* c
c
c     search the number record of last output data
c
      implicit none
c
      integer ik_init, ik, ikmax
      parameter(ikmax=5000)
      real*8 ak_dummy, dummy
c     ------------------------------------------
c
      open(10,file='renormalized_pkreal_2ndBorn.dat',
     &     status='unknown')
c
      do ik=1, ikmax
         read(10,*,END=10) ak_dummy, dummy, dummy
      enddo
c
 10   ik_init = ik
c
      write(6,*) ik_init, ak_dummy
c
      close(10)
c
      end
c
c ************************************************************
c
      SUBROUTINE gauleg(x1,x2,x,w,n)
c
c ************************************************************
c
      INTEGER n
      REAL*8 x1,x2,x(n),w(n)
      DOUBLE PRECISION EPS
      PARAMETER (EPS=3.d-14)
      INTEGER i,j,m
      DOUBLE PRECISION p1,p2,p3,pp,xl,xm,z,z1
      m=(n+1)/2
      xm=0.5d0*(x2+x1)
      xl=0.5d0*(x2-x1)
      do 12 i=1,m
        z=cos(3.141592654d0*(i-.25d0)/(n+.5d0))
1       continue
          p1=1.d0
          p2=0.d0
          do 11 j=1,n
            p3=p2
            p2=p1
            p1=((2.d0*j-1.d0)*z*p2-(j-1.d0)*p3)/j
11        continue
          pp=n*(z*p1-p2)/(z*z-1.d0)
          z1=z
          z=z1-p1/pp
        if(abs(z-z1).gt.EPS)goto 1
        x(i)=xm-xl*z
        x(n+1-i)=xm+xl*z
        w(i)=2.d0*xl/((1.d0-z*z)*pp*pp)
        w(n+1-i)=w(i)
12    continue
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software z!0(0.
c
